package eu.semiramis.federationproxy.saml;

import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;
import org.joda.time.DateTime;
import org.opensaml.common.SAMLObject;
import org.opensaml.common.SAMLObjectBuilder;
import org.opensaml.common.binding.decoding.SAMLMessageDecoder;
import org.opensaml.saml2.binding.decoding.HTTPPostDecoder;
import org.opensaml.saml2.binding.decoding.HTTPRedirectDeflateDecoder;
import org.opensaml.saml2.common.Extensions;
import org.opensaml.saml2.core.Assertion;
import org.opensaml.saml2.core.AttributeQuery;
import org.opensaml.saml2.core.Audience;
import org.opensaml.saml2.core.AudienceRestriction;
import org.opensaml.saml2.core.AuthnRequest;
import org.opensaml.saml2.core.AuthzDecisionQuery;
import org.opensaml.saml2.core.Conditions;
import org.opensaml.saml2.core.EncryptedAssertion;
import org.opensaml.saml2.core.IDPEntry;
import org.opensaml.saml2.core.IDPList;
import org.opensaml.saml2.core.NameID;
import org.opensaml.saml2.core.OneTimeUse;
import org.opensaml.saml2.core.RequestAbstractType;
import org.opensaml.saml2.core.Response;
import org.opensaml.saml2.core.Scoping;
import org.opensaml.saml2.core.Status;
import org.opensaml.saml2.core.StatusCode;
import org.opensaml.saml2.core.StatusMessage;
import org.opensaml.saml2.core.StatusResponseType;
import org.opensaml.saml2.core.Subject;
import org.opensaml.saml2.core.SubjectQuery;
import org.opensaml.saml2.core.impl.ResponseImpl;
import org.opensaml.saml2.encryption.Decrypter;
import org.opensaml.saml2.encryption.EncryptedElementTypeEncryptedKeyResolver;
import org.opensaml.xml.Configuration;
import org.opensaml.xml.XMLObject;
import org.opensaml.xml.XMLObjectBuilderFactory;
import org.opensaml.xml.io.Marshaller;
import org.opensaml.xml.io.MarshallerFactory;
import org.opensaml.xml.security.SecurityHelper;
import org.opensaml.xml.security.credential.BasicCredential;
import org.opensaml.xml.security.keyinfo.StaticKeyInfoCredentialResolver;
import org.opensaml.xml.signature.SignableXMLObject;
import org.opensaml.xml.signature.Signature;
import org.opensaml.xml.signature.SignatureConstants;
import org.opensaml.xml.signature.Signer;
import org.opensaml.xml.signature.impl.SignatureBuilder;
import org.opensaml.xml.util.XMLHelper;

import eu.semiramis.commons.SAMLAttribute;
import eu.semiramis.commons.SAMLUtil;
import eu.semiramis.util.http.HttpUtil;
import eu.semiramis.util.string.StringUtil;

public class SamlManager {

	protected static Logger logger = Logger.getLogger(SamlManager.class);
	
	private static SamlManager samlManagerSingleInstance = null;	
	
	/**
	 * init
	 * @param federationProxyConfigFilePath
	 * @throws Exception
	 */
	public static void init() throws Exception{
		logger.debug("[SamlManager.init()] Start");
		if (samlManagerSingleInstance == null){
			SAMLUtil.startLibrary();
			samlManagerSingleInstance = new SamlManager();
		}
		logger.debug("[SamlManager.init()] End");
	}
	/**
	 * getInstance
	 */
	public static SamlManager getInstance(){
		return samlManagerSingleInstance;
	}
	/**
	 * SamlManager()
	 * @throws Exception
	 */
	private SamlManager(){}

	
	
	
	
	/**
	 * getSamlMessageDecoderInstance
	 * @param requestMethod
	 * @return
	 */
	public SAMLMessageDecoder getSamlMessageDecoderInstance(String requestMethod) {
		SAMLMessageDecoder samlMessageDecoder = null;
		if (HttpUtil.isHttpGetRequest(requestMethod)){
			samlMessageDecoder = new HTTPRedirectDeflateDecoder();
		} else if (HttpUtil.isHttpPostRequest(requestMethod)){
			samlMessageDecoder = new HTTPPostDecoder();
		}
		return samlMessageDecoder;
	}
	public String extractSamlObjectId(SAMLObject samlObject) {
		String samlObjectId = null;
		if (samlObject != null){
			if (samlObject instanceof AuthnRequest){
				samlObjectId = ((AuthnRequest)samlObject).getID();
			} else if (samlObject instanceof SubjectQuery){
				samlObjectId = ((SubjectQuery)samlObject).getID();
			} else if (samlObject instanceof ResponseImpl){
				samlObjectId = ((ResponseImpl)samlObject).getID();
			}
		}
		return samlObjectId;
	}
	

    public List<Assertion> extractSamlAssertionList(SAMLObject samlObject) throws Exception{
    	List<Assertion> assertionList = null;
    	if (samlObject != null && samlObject instanceof RequestAbstractType){
    		RequestAbstractType requestAbstractType = (RequestAbstractType) samlObject;
    		Extensions extensions = requestAbstractType.getExtensions();
    		if (extensions != null){
    			assertionList = new ArrayList<Assertion>(); 
    			List<Assertion> recoveredAssertionList = SAMLUtil.recoverAssertions(extensions);
    			Iterator<Assertion> it = recoveredAssertionList.iterator();
    			while (it.hasNext()) {
    				Assertion assertion = it.next();
    				//if (!assertion.getAuthnStatements().isEmpty()){
    				//	SAMLUtil.releaseAssertion(assertion);
    				//}
    				//if(!assertion.getAttributeStatements().isEmpty()){
    				//	SAMLUtil.releaseAssertion(assertion);
    				//}
    				//if(!assertion.getAuthzDecisionStatements().isEmpty()){
    				//	SAMLUtil.releaseAssertion(assertion);
    				//}
    				assertionList.add(assertion);    				
    			}
    		}
    	} else if (samlObject != null && samlObject instanceof ResponseImpl){
    		ResponseImpl responseImpl = (ResponseImpl) samlObject;
    		//ReleaseAssertions
    		List<Assertion> responseImplAssertionList = responseImpl.getAssertions();
    		if (responseImplAssertionList != null){
    			if (assertionList == null){
    				assertionList = new ArrayList<Assertion>();
    			}
    			assertionList.addAll(SAMLUtil.getAssertions(responseImpl));
                Iterator<Assertion> assertionIt = assertionList.iterator();
                while (assertionIt.hasNext()) {
                    Assertion assertion = assertionIt.next();
                    //SAMLUtil.releaseAssertion(assertion);
                }
    		}
    	}
        return assertionList;
    }
	
	
    public List<EncryptedAssertion> extractSamlEncryptedAssertionList(SAMLObject samlObject) throws Exception{
    	List<EncryptedAssertion> encryptedAssertionList = new ArrayList<EncryptedAssertion>();
    	if (samlObject != null && samlObject instanceof ResponseImpl){
    		ResponseImpl responseImpl = (ResponseImpl) samlObject;
    		List<EncryptedAssertion> responseImplEncryptedAssertionList = responseImpl.getEncryptedAssertions();
    		if (responseImplEncryptedAssertionList != null){
    			encryptedAssertionList = responseImplEncryptedAssertionList;
    	    }
    	}
        return encryptedAssertionList;
    }
    
    /*
    public List<Assertion> getSourceSamlDecryptedReleasedAssertionList(SAMLObject samlObject) throws Exception{
    	List<Assertion> assertionList = null;
    	if (samlObject instanceof RequestAbstractType){
    		RequestAbstractType requestAbstractType = (RequestAbstractType) samlObject;
    		Extensions extensions = requestAbstractType.getExtensions();
    		if (extensions != null){
    			assertionList = new ArrayList<Assertion>(); 
    			List<Assertion> recoveredAssertionList = SAMLUtil.recoverAssertions(extensions);
    			Iterator<Assertion> it = recoveredAssertionList.iterator();
    			while (it.hasNext()) {
    				Assertion assertion = it.next();
    				if (!assertion.getAuthnStatements().isEmpty()){
    					SAMLUtil.releaseAssertion(assertion);
    				}
    				if(!assertion.getAttributeStatements().isEmpty()){
    					SAMLUtil.releaseAssertion(assertion);
    				}
    				if(!assertion.getAuthzDecisionStatements().isEmpty()){
    					SAMLUtil.releaseAssertion(assertion);
    				}
    				assertionList.add(assertion);    				
    			}
    		}
    	} else if (samlObject instanceof ResponseImpl){
    		ResponseImpl responseImpl = (ResponseImpl) samlObject;
    		//ReleaseAssertions
    		List<Assertion> responseImplAssertionList = responseImpl.getAssertions();
    		if (responseImplAssertionList != null){
    			if (assertionList == null){
    				assertionList = new ArrayList<Assertion>();
    			}
    			assertionList.addAll(SAMLUtil.getAssertions(responseImpl));
                Iterator<Assertion> assertionIt = assertionList.iterator();
                while (assertionIt.hasNext()) {
                    Assertion assertion = assertionIt.next();
                    SAMLUtil.releaseAssertion(assertion);
                }
    		}
    		List<EncryptedAssertion> responseImplEncryptedAssertionList = responseImpl.getEncryptedAssertions();
    		if (responseImplEncryptedAssertionList != null){
    			if (assertionList == null){
    				assertionList = new ArrayList<Assertion>();
    			}
                Iterator<EncryptedAssertion> encryptedAssertionIt = responseImplEncryptedAssertionList.iterator();
                while (encryptedAssertionIt.hasNext()) {
                    EncryptedAssertion encryptedAssertion = encryptedAssertionIt.next();
                    Assertion decryptedAssertion = FederationProxySamlManager.decryptAssertion(encryptedAssertion, getPkcsPublicKey(), getPkcsPrivateKey());
                    assertionList.add(decryptedAssertion);
                }
    	    }
    	}
        return assertionList;
    }
    */

    
    
	/**
	 * 
	 * extractSamlIssuerValue
	 * @param samlObject
	 * @return
	 * @throws Exception
	 */
	public String extractSamlIssuerValue(SAMLObject samlObject) throws Exception{
		String samlIssuerValue = null;
		if (samlObject instanceof RequestAbstractType){
			RequestAbstractType samlRequest = (RequestAbstractType) samlObject;
			samlIssuerValue = samlRequest.getIssuer().getValue();
		} else if (samlObject instanceof Response) {
			StatusResponseType samlResponse = (StatusResponseType) samlObject;
			samlIssuerValue = samlResponse.getIssuer().getValue();
		}
		return samlIssuerValue;
	}
		
	/**
	 * insertSamlIdpProviderId
	 * @param authnRequest
	 * @param samlProviderId
	 */
	public void insertSamlIdpProviderId(AuthnRequest authnRequest, String providerId){
		SAMLObjectBuilder samlObjectBuilder = null;
		if (providerId != null){
			XMLObjectBuilderFactory xmlObjectBuilderFactory = Configuration.getBuilderFactory();
			samlObjectBuilder = (SAMLObjectBuilder) xmlObjectBuilderFactory.getBuilder(Scoping.DEFAULT_ELEMENT_NAME);
			Scoping scoping = (Scoping) samlObjectBuilder.buildObject();
			samlObjectBuilder = (SAMLObjectBuilder) xmlObjectBuilderFactory.getBuilder(IDPList.DEFAULT_ELEMENT_NAME);
			IDPList idpList = (IDPList) samlObjectBuilder.buildObject();
			samlObjectBuilder = (SAMLObjectBuilder) xmlObjectBuilderFactory.getBuilder(IDPEntry.DEFAULT_ELEMENT_NAME);
			IDPEntry idpEntry = (IDPEntry) samlObjectBuilder.buildObject();
			idpEntry.setProviderID(providerId);
			idpList.getIDPEntrys().add(idpEntry);
			scoping.setIDPList(idpList);
			authnRequest.setScoping(scoping);
		}
	}
	/**
	 * extractSamlIdpProviderId samlIdpProviderId is the saml2 service provider id stored into the AuthnRequest
	 * @param samlRequest
	 * @return
	 */
	public String extractSamlIdpProviderId(SAMLObject samlObject) {
		String samlIdpProviderId = null;
		if (samlObject != null && samlObject instanceof AuthnRequest){
			samlIdpProviderId = extractSamlIdpProviderId((AuthnRequest)samlObject);
		}
		return samlIdpProviderId;
	}
	public String extractSamlIdpProviderId(AuthnRequest authnRequest){
		String samlIdpProviderId = null;
		Scoping scoping = authnRequest.getScoping();
		if (scoping != null){
			IDPList idpList = scoping.getIDPList();
			if (idpList != null){
				List<IDPEntry> idpEntrys = idpList.getIDPEntrys();
				if (idpEntrys != null && idpEntrys.size() > 0){
					IDPEntry idpEntry = idpEntrys.get(0);
					if (idpEntry != null){
						samlIdpProviderId = idpEntry.getProviderID();
					}
				}
			}
		}
		return samlIdpProviderId;
	}
	/**
	 * extractSamlSubjectNameQualifier
	 * @param samlRequest
	 * @return
	 */
	public String extractSamlSubjectNameQualifier(SAMLObject samlObject){
		String subjectNameQualifier = null;
		if (samlObject != null && samlObject instanceof SubjectQuery){
			subjectNameQualifier = extractSamlSubjectNameQualifier(((SubjectQuery)samlObject));
		}
		return subjectNameQualifier;
	}
	public String extractSamlSubjectNameQualifier(SubjectQuery subjectQuery){
		String subjectNameQualifier = null;
		Subject subject = subjectQuery.getSubject();
		if (subject != null){
			NameID nameID = subject.getNameID();
			if (nameID != null){
				subjectNameQualifier = nameID.getNameQualifier();
			}
		}
		return subjectNameQualifier;
	}

	/**
	 * extractSamlSubjectNameId is the saml2 service provider id stored into the AuthnRequest
	 * @param samlRequest
	 * @return
	 */
	public String extractSamlSubjectNameId(SAMLObject samlObject){
		String subjectNameIdValue = null;
		if (samlObject != null && samlObject instanceof SubjectQuery){
			subjectNameIdValue = extractSamlSubjectNameId((SubjectQuery) samlObject);
		}
		return subjectNameIdValue;
	}
	public String extractSamlSubjectNameId(SubjectQuery subjectQuery){
		String subjectNameIdValue = null;
		Subject subject = subjectQuery.getSubject();
		if (subject != null){
			NameID nameID = subject.getNameID();
			if (nameID != null){
				subjectNameIdValue = nameID.getValue();
			}
		}
		return subjectNameIdValue;
	}
	
	/**
	 * extractSamlResponseStatusCode
	 * @param samlResponse
	 * @return
	 */
	public String extractSamlResponseStatusCode(SAMLObject samlObject){
		String samlResponseStatusCode = null;
		if (samlObject != null && samlObject instanceof ResponseImpl){
			samlResponseStatusCode = extractSamlResponseStatusCode((ResponseImpl)samlObject);
		}
		return samlResponseStatusCode;
	}
	public String extractSamlResponseStatusCode(StatusResponseType samlResponse){
		String samlResponseStatusCode = null;
		Status status = samlResponse.getStatus();
		if (status != null){
			StatusCode statusCode = status.getStatusCode();
			if (statusCode != null){
				samlResponseStatusCode = statusCode.getValue();
			}
		}
		return samlResponseStatusCode;
	}
	
	/**
	 * extractSamlResponseStatusMessage
	 * @param samlResponse
	 * @return
	 */
	public String extractSamlResponseStatusMessage(SAMLObject samlObject){
		String samlResponseStatusMessage = null;
		if (samlObject != null && samlObject instanceof ResponseImpl){
			samlResponseStatusMessage = extractSamlResponseStatusMessage((ResponseImpl)samlObject);
		}
		return samlResponseStatusMessage;
	}
	public String extractSamlResponseStatusMessage(StatusResponseType samlResponse){
		String samlResponseStatusMessage = null;
		Status status = samlResponse.getStatus();
		if (status != null){
			StatusMessage statusMessage = status.getStatusMessage();
			if (statusMessage != null){
				samlResponseStatusMessage = statusMessage.getMessage();
			}
		}
		return samlResponseStatusMessage;
	}
	
	/**
	 * insertSamlOneTimeUse
	 * @param authnRequest
	 * @param oneTimeUseFlag
	 */
	public void insertSamlConditions(AuthnRequest authnRequest, Boolean oneTimeUseFlag, DateTime notBefore, DateTime notOnAfter, List<String> audienceRestrictionIdList){
		SAMLObjectBuilder auxSamlObjectBuilder;
		if ((oneTimeUseFlag != null && oneTimeUseFlag.booleanValue()) ||
			notBefore != null ||
			notOnAfter != null ||
			(audienceRestrictionIdList != null && audienceRestrictionIdList.size() > 0)){
			//Init conditions
			XMLObjectBuilderFactory xmlObjectBuilderFactory = Configuration.getBuilderFactory();
			auxSamlObjectBuilder = (SAMLObjectBuilder) xmlObjectBuilderFactory.getBuilder(Conditions.DEFAULT_ELEMENT_NAME);
			Conditions conditions = (Conditions) auxSamlObjectBuilder.buildObject();
			if (oneTimeUseFlag != null && oneTimeUseFlag.booleanValue()){
				auxSamlObjectBuilder = (SAMLObjectBuilder) xmlObjectBuilderFactory.getBuilder(OneTimeUse.DEFAULT_ELEMENT_NAME);
				OneTimeUse oneTimeUse = (OneTimeUse) auxSamlObjectBuilder.buildObject();
				conditions.getConditions().add(oneTimeUse);
			}
			if (notBefore != null){
				conditions.setNotBefore(notBefore);
			}
			if (notOnAfter != null){
				conditions.setNotOnOrAfter(notOnAfter);
			}
			if (audienceRestrictionIdList != null && audienceRestrictionIdList.size() > 0){
				Iterator<String> iterator = audienceRestrictionIdList.iterator();
				while (iterator.hasNext()) {
					String audienceRestrictionId = iterator.next();
					List<AudienceRestriction> audienceRestrictions = conditions.getAudienceRestrictions();
					auxSamlObjectBuilder = (SAMLObjectBuilder) xmlObjectBuilderFactory.getBuilder(AudienceRestriction.DEFAULT_ELEMENT_NAME);
					AudienceRestriction audienceRestriction = (AudienceRestriction) auxSamlObjectBuilder.buildObject();
					auxSamlObjectBuilder = (SAMLObjectBuilder) xmlObjectBuilderFactory.getBuilder(Audience.DEFAULT_ELEMENT_NAME);
					Audience audience = (Audience) auxSamlObjectBuilder.buildObject();
					audience.setAudienceURI(audienceRestrictionId);
					audienceRestriction.getAudiences().add(audience);
					conditions.getAudienceRestrictions().add(audienceRestriction);
				}
			}

			authnRequest.setConditions(conditions);			
		}
	}
	/**
	* insertSamlOneTimeUse
	 * @param authnRequest
	 * @param oneTimeUseFlag
	 */
	public void insertSamlConditions(AuthnRequest authnRequest, Conditions conditions){
		Boolean oneTimeUse = extractSamlOneTimeUseFlag(conditions);
		DateTime notBefore = extractSamlNotBefore(conditions);
		DateTime notOnAfter = extractSamlNotOnAfter(conditions);
		List<String> audienceRestrictionIdList = extractSamlAudienceRestrictionIdList(conditions);
		insertSamlConditions(authnRequest, oneTimeUse, notBefore, notOnAfter, audienceRestrictionIdList);
	}
	
	/**
	 * extractSamlOneTimeUseFlag
	 * @param samlRequest
	 * @return
	 */
	public Boolean extractSamlOneTimeUseFlag(Conditions conditions){
		Boolean oneTimeUseFlag = null;
		//Case SAML2 "AuthnRequest"
		if (conditions != null){
			OneTimeUse oneTimeUse = conditions.getOneTimeUse();
			if (oneTimeUse != null){
				oneTimeUseFlag = new Boolean(true);
			}
		}
		return oneTimeUseFlag;
	}

	public String extractSamlInResponseTo(SAMLObject samlObject) {
		String samlInResponseTo = null;
		if (samlObject != null && samlObject instanceof ResponseImpl){
			samlInResponseTo = ((ResponseImpl)samlObject).getInResponseTo();
		}
		return samlInResponseTo;
	}

    public String extractSamlActionId(SAMLObject samlObject) {
    	String samlActionId = null;
        if(samlObject instanceof AuthzDecisionQuery){
        	samlActionId = ((AuthzDecisionQuery)samlObject).getActions().get(0).getAction();
        }
        return samlActionId;
    }
	
    public String extractSamlResourceId(SAMLObject samlObject) {
    	String samlResourceId = null;
        if(samlObject != null && samlObject instanceof AuthzDecisionQuery){
        	samlResourceId = ((AuthzDecisionQuery)samlObject).getResource();
        }
        return samlResourceId;
    }
    
    /**
     * @deprecated Replaced by extractSamlQueryAttributes(SAMLObject samlObject).
     */
	public List<String> extractSamlQueriedAttributes(SAMLObject samlObject) {
    	List<String> attributeList = null;
        if(samlObject != null && samlObject instanceof AttributeQuery){
        	attributeList = SAMLUtil.getQueriedAttributes((AttributeQuery)samlObject);
        }
        return attributeList;
    }
	
	public List<SAMLAttribute> extractSamlQueryAttributes(SAMLObject samlObject) {
    	List<SAMLAttribute> attributeList = null;
        if(samlObject != null && samlObject instanceof AttributeQuery){
        	attributeList = SAMLUtil.getQueryAttributes((AttributeQuery)samlObject);
        }
        return attributeList;
    }

	

	
	
	/**
	 * extractSamlConditions
	 * @param samlRequest
	 * @return
	 */
	public Conditions extractSamlConditions(SAMLObject samlObject){
		Conditions conditions = null;
		if (samlObject != null && samlObject instanceof AuthnRequest){
			conditions = extractSamlConditions((AuthnRequest)samlObject);
		}
		return conditions;
	}
	
	public Conditions extractSamlConditions(AuthnRequest authnRequest){
		Conditions conditions = authnRequest.getConditions();
		return conditions;
	}
	/**
	 * extractSamlNotBefore
	 * @param samlRequest
	 * @return
	 */
	public DateTime extractSamlNotBefore(Conditions conditions){
		DateTime notBefore = null;
		if (conditions != null){
			notBefore = conditions.getNotBefore();	
		}
		return notBefore;
	}
	/**
	 * extractSamlNotOnAfter
	 * @param samlRequest
	 * @return
	 */
	public DateTime extractSamlNotOnAfter(Conditions conditions){
		DateTime notOnAfter = null;
		if (conditions != null){
			notOnAfter = conditions.getNotOnOrAfter();
		}
		return notOnAfter;
	}
	/**
	 * extractSamlAudienceRestrictionIdList
	 * @param samlRequest
	 * @return
	 */
	public List<String> extractSamlAudienceRestrictionIdList(Conditions conditions){
		List<String> audienceRestrictionIdList = new ArrayList<String>();
		if (conditions != null){
			List<AudienceRestriction> audienceRestrictionsList = conditions.getAudienceRestrictions();
			if (audienceRestrictionsList != null){
				Iterator<AudienceRestriction> iteratorAudienceRestriction = audienceRestrictionsList.iterator();
				while (iteratorAudienceRestriction.hasNext()) {
					AudienceRestriction audienceRestriction = iteratorAudienceRestriction.next();
					List<Audience> audienceList = audienceRestriction.getAudiences();
					if (audienceList != null){
						Iterator<Audience> iteratorAudience = audienceList.iterator();
						while (iteratorAudience.hasNext()) {
							Audience audience = iteratorAudience.next();
							String audienceRestrictionId = audience.getAudienceURI();
							if (audienceRestrictionId != null){
								audienceRestrictionIdList.add(audienceRestrictionId);
							}
						}
					}
				}
			}
		}
		if (audienceRestrictionIdList.size() == 0){
			audienceRestrictionIdList = null;
		}
		return audienceRestrictionIdList;
	}
	
	/**
	 * extractSamlLocalPart.- SamlLocalPart stores the samlRequestType ["AuthnRequest"]["AttributeQuery"]["AuthorizationQuery"] 
	 * @param samlRequest
	 * @return
	 */
	public String extractSamlLocalPart(RequestAbstractType samlRequest){
		String samlLocalPart = samlRequest.getElementQName().getLocalPart();
		return samlLocalPart;
	}
	
	
	/**
	 * signXML
	 */
	public void signSamlObject(SignableXMLObject xmlSignableObject, PublicKey pkcsPublicKey, PrivateKey pkcsPrivateKey) throws Exception {
		XMLObjectBuilderFactory builderFactory = Configuration.getBuilderFactory();
		SignatureBuilder signatureBuilder = (SignatureBuilder) builderFactory.getBuilder(Signature.DEFAULT_ELEMENT_NAME);
		Signature signature = signatureBuilder.buildObject();
		BasicCredential basicCredential = SecurityHelper.getSimpleCredential(pkcsPublicKey, pkcsPrivateKey);
		signature.setSigningCredential(basicCredential);
		signature.setCanonicalizationAlgorithm(SignatureConstants.ALGO_ID_C14N_EXCL_OMIT_COMMENTS);
		signature.setSignatureAlgorithm(SignatureConstants.ALGO_ID_SIGNATURE_RSA_SHA1);
		xmlSignableObject.setSignature(signature);
		MarshallerFactory marshallerFactory = Configuration.getMarshallerFactory();
		Marshaller marshaller = marshallerFactory.getMarshaller(xmlSignableObject);
		marshaller.marshall(xmlSignableObject);
		Signer.signObject(signature);
	}

	
    public Assertion decryptAssertion(EncryptedAssertion encryptedAssertion, PublicKey publicKey, PrivateKey privateKey) throws Exception {
   		org.opensaml.xml.security.credential.BasicCredential credential = SecurityHelper.getSimpleCredential(publicKey, privateKey);
    	StaticKeyInfoCredentialResolver skicr = new StaticKeyInfoCredentialResolver(credential);
    	Decrypter samlDecrypter = new Decrypter(null, skicr, new EncryptedElementTypeEncryptedKeyResolver());
    	Assertion assertion = samlDecrypter.decrypt(encryptedAssertion);
    	return assertion;
    }
	

	public String extractSamlAssertionConsumerServiceUrl(SAMLObject samlObject) {
		String samlAssertionConsumerServiceUrl = null;
		if (samlObject != null && samlObject instanceof AuthnRequest){
			samlAssertionConsumerServiceUrl = ((AuthnRequest)samlObject).getAssertionConsumerServiceURL();
		}
		return samlAssertionConsumerServiceUrl;
	}
	
    
	/**
	 * toConsole
	 * @param requestAbstractType
	 */
	/*
	public void toConsole(RequestAbstractType requestAbstractType){
		System.out.println("requestAbstractType.getConsent(): " + requestAbstractType.getConsent()); 
		System.out.println("requestAbstractType.getDestination(): " + requestAbstractType.getDestination());
		System.out.println("requestAbstractType.getID(): " + requestAbstractType.getID());
		System.out.println("requestAbstractType.getIDIndex(): " + requestAbstractType.getIDIndex());
		System.out.println("requestAbstractType.getIssuer().getValue(): " + requestAbstractType.getIssuer().getValue());
		System.out.println("requestAbstractType.getSignatureReferenceID(): " + requestAbstractType.getSignatureReferenceID());
	}
	*/

	
	
	public static final int SAMLTYPE_NONE				= 0;
	public static final int SAMLTYPE_AUTHNREQUEST		= 1;
	public static final int SAMLTYPE_ATTRIBUTEQUERY		= 2;
	public static final int SAMLTYPE_AUTHDECISIONQUERY	= 3;
	public static final int SAMLTYPE_RESPONSEIMPL		= 4;

	public int getSamlObjectType(SAMLObject samlObject) {
		int samlObjectType = SAMLTYPE_NONE;
		if (samlObject != null){
			if (samlObject instanceof AuthnRequest){
				samlObjectType = SAMLTYPE_AUTHNREQUEST;
			} else if (samlObject instanceof AttributeQuery){
				samlObjectType = SAMLTYPE_ATTRIBUTEQUERY;
			} else if (samlObject instanceof AuthzDecisionQuery){
				samlObjectType = SAMLTYPE_AUTHDECISIONQUERY;
			} else if (samlObject instanceof ResponseImpl){
				samlObjectType = SAMLTYPE_RESPONSEIMPL;
			}
		}
		return samlObjectType;
	}

	
	private String samlObjectType_toString(int samlObjectType){
		String samlObjectTypeStream = null;
		switch (samlObjectType){
			case SAMLTYPE_AUTHNREQUEST:
				samlObjectTypeStream = AuthnRequest.class.getName();
				break;
			case SAMLTYPE_ATTRIBUTEQUERY:
				samlObjectTypeStream = AttributeQuery.class.getName();
				break;
			case SAMLTYPE_AUTHDECISIONQUERY:
				samlObjectTypeStream = AuthzDecisionQuery.class.getName();
				break;
			case SAMLTYPE_RESPONSEIMPL:
				samlObjectTypeStream = ResponseImpl.class.getName();
				break;
		}
		return samlObjectTypeStream;
	}

	
	public String toString(SAMLObject samlObject){
		StringBuffer sb = null;
		try {
			sb = new StringBuffer();
			sb.append("\n");
			sb.append("\t");sb.append("[");sb.append("SAMLObject");sb.append("]");sb.append("\n");
			sb.append("\t\t");sb.append("[");sb.append("samlObjectId: ");sb.append(extractSamlObjectId(samlObject));sb.append("]");sb.append("\n");
			sb.append("\t\t");sb.append("[");sb.append("samlObjectType: ");sb.append(getSamlObjectType(samlObject) + "-" + samlObjectType_toString(getSamlObjectType(samlObject)));sb.append("]");sb.append("\n");
			sb.append("\t\t");sb.append("[");sb.append("samlIssuer: ");sb.append(extractSamlIssuerValue(samlObject));sb.append("]");sb.append("\n");
			sb.append("\t\t");sb.append("[");sb.append("AuthenticationParameters");sb.append("]");sb.append("\n");
			sb.append("\t\t\t");sb.append("[");sb.append("samlIdpProvider: ");sb.append(extractSamlIdpProviderId(samlObject));sb.append("]");sb.append("\n");
			sb.append("\t\t\t");sb.append("[");sb.append("samlAssertionConsumerServiceUrl: ");sb.append(extractSamlAssertionConsumerServiceUrl(samlObject));sb.append("]");sb.append("\n");
			Conditions samlConditions = extractSamlConditions(samlObject);
			sb.append("\t\t\t");sb.append("[");sb.append("samlOneTimeUseFlag: ");sb.append(extractSamlOneTimeUseFlag(samlConditions));sb.append("]");sb.append("\n");
			sb.append("\t\t\t");sb.append("[");sb.append("samlNotBefore: ");sb.append(extractSamlNotBefore(samlConditions));sb.append("]");sb.append("\n");
			sb.append("\t\t\t");sb.append("[");sb.append("samlNotOnAfter: ");sb.append(extractSamlNotOnAfter(samlConditions));sb.append("]");sb.append("\n");
			sb.append("\t\t\t");sb.append("[");sb.append("samlAudienceRestrictionIdList: ");sb.append(extractSamlAudienceRestrictionIdList(samlConditions));sb.append("]");sb.append("\n");
			sb.append("\t\t");sb.append("[");sb.append("subjectQueryParameters");sb.append("]");sb.append("\n");			
			sb.append("\t\t\t");sb.append("[");sb.append("samlSubjectNameId: ");sb.append(extractSamlSubjectNameId(samlObject));sb.append("]");sb.append("\n");
			sb.append("\t\t\t");sb.append("[");sb.append("samlSubjectNameQualifier: ");sb.append(extractSamlSubjectNameQualifier(samlObject));sb.append("]");sb.append("\n");			
			sb.append("\t\t");sb.append("[");sb.append("sourceAuthorizationParameters");sb.append("]");sb.append("\n");
			sb.append("\t\t\t");sb.append("[");sb.append("samlActionId: ");sb.append(extractSamlActionId(samlObject));sb.append("]");sb.append("\n");
			sb.append("\t\t\t");sb.append("[");sb.append("samlResourceId: ");sb.append(extractSamlResourceId(samlObject));sb.append("]");sb.append("\n");
			sb.append("\t\t");sb.append("[");sb.append("sourceAttributeQueryParameters");sb.append("]");sb.append("\n");
			sb.append("\t\t\t");sb.append("[");sb.append("samlQueriedAttributes: ");sb.append(extractSamlQueriedAttributes(samlObject));sb.append("]");sb.append("\n");
			sb.append("\t\t");sb.append("[");sb.append("targetResponseParameters");sb.append("]");sb.append("\n");
			sb.append("\t\t\t");sb.append("[");sb.append("samlInResponseTo: ");sb.append(extractSamlInResponseTo(samlObject));sb.append("]");sb.append("\n");
			sb.append("\t\t\t");sb.append("[");sb.append("samlResponseStatusCode: ");sb.append(extractSamlResponseStatusCode(samlObject));sb.append("]");sb.append("\n");			
			sb.append("\t\t");sb.append("[");sb.append("samlObject");sb.append("]");sb.append("\n");sb.append(toXml(samlObject));;sb.append("\n");
		} catch (Exception e){
			sb = new StringBuffer();
			sb.append("ToStringException: " + e.getMessage());
		}
		return sb.toString();
	}

    public String toXml(XMLObject xmlObject){
    	String xmlObjectStream = null;
    	try {
    		MarshallerFactory marshallerFactory = Configuration.getMarshallerFactory();
    		Marshaller marshaller = marshallerFactory.getMarshaller(xmlObject);
            org.w3c.dom.Element subjectElement = marshaller.marshall(xmlObject);
            xmlObjectStream = XMLHelper.nodeToString(subjectElement);
            xmlObjectStream = StringUtil.replace(xmlObjectStream, "><", ">\n<");
        } catch(Exception e) {
        	xmlObjectStream = "[ToXmlException: " + e.getMessage() + "]";
        }
        return xmlObjectStream;
    }

	

}
